function [perR, LrR, RatioOnes,T] = bruteForceIterSol(Y, nbrOfPeriods, PnPrior,chi2limit,silent)
%[perR, LrR, RatioOnes,T] = bruteForceIterSol(Y, nbrOfPeriods, PnPrior,chi2limit,silent)
%   Estimates the periodicies in a sequence of zeros and ones.
% Input:
%       Y -input sequence containing zeros and ones (B,N) where B is the
%       alphabet size and N the lenght of the sequence
%       nbrOfPeriodes - the highest periodicity one wants to find
%       PnPrior - best value is 0.5. Makes sure the algorithm finds periodicities
%               that is based on the ones and not the zeros.
%       chi2limit -  the statistical threshold. A good choice is chi2limit =  14.5848
%       silent - silent mode or not
% Output:
%       perR - The found periodicies
%       LrR  - The value of the likelihood test
%       RatioOnes - The ratio of ones in the sequence (meaningless) 
%       T - matrix containing all periodicities (meaningless).

% Johan Sw?rd & Stefan Ingi Adalbj?rnsson, Oct 2014

perR=zeros(nbrOfPeriods,1);
LrR=perR;
RatioOnes=LrR;
nbrOfSymbols = size(Y,1);
for ii = 1:nbrOfSymbols
    clear x y T index2 index ind
    y = Y(ii,:);
    periods = 1:nbrOfPeriods;
    T = zeros(length(y),sum(periods));
    T(:,1) = ones(length(y),1);
    pos=2;
    for iii = 3:length(periods)
        I = eye(periods(iii));
        IM = repmat(I,length(y),1);
        T(:,pos:pos+periods(iii)-1) = IM(1:length(y),:);
        pos = pos + periods(iii);
    end
    [~,Tp]=size(T);
    for counter=1:Tp
        Itmp=find(T(:,counter));
        SumY(counter)=sum(Y(Itmp)); %Counting number of 1 in y
        SumT(counter)=sum(T(:,counter)); % Counting number of 1 in T
    end
    lengthy=length(y);
    sumy=sum(y);
    
    
    
    counter=1;
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% H0 and H1 have unkown parameters Pp and Pn
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PpH0=sumy/lengthy;
itermax=5;
TH0=zeros(lengthy,1);
delta=1e-88;
for iter=1:itermax
    for counter=1:Tp
        
        % H0
        IpH0=find(TH0);
        IpCompH0=find(TH0==0);
        
        PpH0=sum(y(IpCompH0))/length(y(IpCompH0));
        
        PnH0=max((length(y(IpH0))-sum(y(IpH0)))/length(y(IpCompH0)),0.000001);
        
        LLH0(counter)=sum(y(IpCompH0))*log(PpH0+delta)+(length(y(IpCompH0))-...
            sum(y(IpCompH0)))*log(1-PpH0+delta)+...
            sum(y(IpH0))*log(1-PnH0+delta)+(length(y(IpH0))-sum(y(IpH0)))*log(PnH0+delta);
        
        % H1
        TH1=TH0+T(:,counter);
        IpH1=find(TH1);
        IpCompH1=find(TH1==0);
        if isempty(IpCompH1)~=1
            PpH1=sum(y(IpCompH1))/length(y(IpCompH1));
            PpH1=min(0.5,PpH1);
        else
            PpH1=delta;
        end
        PnH1=(length(y(IpH1))-sum(y(IpH1)))/length(y(IpH1));
        
        
        if PnPrior~=0
            % sum(y(find(T(:,counter))))
            PnH1=min(PnH1,PnPrior);
            
        end
        
        
        LLH1(counter)=sum(y(IpCompH1))*log(PpH1+delta)+(length(y(IpCompH1))-...
            sum(y(IpCompH1)))*log(1-PpH1+delta)+...
            sum(y(IpH1))*log(1-PnH1+delta)+(length(y(IpH1))-sum(y(IpH1)))*log(PnH1+delta);%+...
    end
    
    Indcounter=1;
    Results=zeros(nbrOfPeriods,nbrOfPeriods);
    DstatUnKnown=Results;
    for j = [1, 3:nbrOfPeriods]
        for i = 1:j
            Results(i,j)=SumY(Indcounter)/SumT(Indcounter);
            DstatUnKnown(i,j)=-2*(LLH0(Indcounter)-LLH1(Indcounter));
            %            MeshPp(i,j)=PpH1(counter);
            %            MeshPn(i,j)=PnH1(counter);
            Indcounter=Indcounter+1;
        end
    end
    [Y,Idstat] = max(DstatUnKnown(:));
    [tmpPhase,tmpPer] = ind2sub(size(DstatUnKnown),Idstat);
    if max(-2*(LLH0-LLH1)) > chi2limit;
        [LrR(iter),ImaxH1]=max(-2*(LLH0-LLH1));
        
        RatioOnes=mean(y(find(T(:,ImaxH1))));
        
        
        
        %  if Results(Idstat) > MinPerc
             
        if min(TH0-T(:,ImaxH1))< 0
            perR(iter)=tmpPer;
             TH0=TH0+T(:,ImaxH1);
        else
            return
        end
        % else
        %   iter=itermax;
        %end
    elseif chi2limit~=0
        return;
    end
    if silent == 0
        figure
        plot(max(DstatUnKnown))
        perR
        % if iter==1
        figure
        %mesh(X,Y,Results)
        plot(max(Results))
        sum(y(find(T(:,ImaxH1))))
        figure
        [X,Y] = meshgrid(1:nbrOfPeriods,1:nbrOfPeriods);
        mesh(X,Y,DstatUnKnown)
        title(['per chosen: ' num2str(perR(iter)) ' andel ettor :' num2str(sum(y(find(T(:,ImaxH1))))/length(y(find(T(:,ImaxH1)))))])
        % end
    end
end
end
%[X,Y] = meshgrid(1:nbrOfPeriods,1:nbrOfPeriods)
%mesh(X,Y,DstatUnKnown)


